//
//  AddPaymentMethodSuccessRunnable.m
//  WebtehPaymentLib
//
//  Created by Jasmin Suljic on 04/10/2018.
//

#import <Foundation/Foundation.h>
#import "AddPaymentMethodSuccessRunnable.h"
#import "RedirectResolver.h"
#import "PaymentMethod.h"
#import "InputResolver.h"

@interface AddPaymentMethodSuccessRunnable ()
    @property(nonatomic, weak) id<AddPaymentMethodLifecycleDelegate> delegate;
@end

@implementation AddPaymentMethodSuccessRunnable

- (id)init:(id<AddPaymentMethodLifecycleDelegate>)delegate {
    _delegate = delegate;
    return [self init];
}

- (void)run:(int)resolvedCode request:(NSString *)request redirectResolver:(RedirectResolver *)redirectResolver inputParam:(NSString *)inputParam {
    // If transaction is not in progress there is no need for transaction success handling
    // This is guard from multiple delegate calls when timeout is invoked and webview is not stopped.
    // After a while transaction success or transaction failure occurs.
    if (!redirectResolver.transactionInProgress) {
        [redirectResolver logTransactionInfo:@"Transaction success invoked, but transactionInProgress = false. Ignoring."];
        return;
    }
    
    redirectResolver.transactionInProgress = false;
    [redirectResolver stopTimeoutTracking];
    [_delegate hideProgress];
    
    PaymentMethod* method = [PaymentMethod fromJson:[InputResolver decode:inputParam]];
    
    [redirectResolver logTransactionInfo:[NSString stringWithFormat:@"Payment method success %@", method]];
    
    [_delegate addPaymentMethodSuccess:method];
}

@end
