//
//  PgwApi.m
//  pay
//
//  Created by Jasmin Suljic on 21/12/16.
//  Copyright © 2016 Leftor d.o.o. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "PgwApi.h"
#import <AFNetworking/AFNetworking.h>
#define API_URL @"https://mobile.webteh.hr/pgw"
//#define API_URL @"http://localhost/pgw"
#define API_AUTH_KEY_QUERY @"auth_token"

@interface PgwApi()

@property(strong,nonatomic) NSString *authToken;
@property bool isDebug;


+(ApiError *) unableToParseResponse;
+(ApiError *) unableToParseErrorResponse;
+(ApiError *) getErrorFromFailure: (NSError *) error;

@end

@implementation PgwApi

-(id)init:(NSString *) authToken
  isDebug:(bool) isDebug {
    self = [super init];

    if (self) {
        self.authToken = authToken;
        self.isDebug = isDebug;
    }

    return self;
}

-(void) checkOrderId:(ShowOrderModel *)model success:(void (^)(ShowOrderResponse *))successCallback failure:(void (^)(ApiError *))failureCallback{
    AFHTTPSessionManager *manager = [AFHTTPSessionManager manager];
    [manager GET:[NSString stringWithFormat:@"%@/api/check-order-id?order_number=%@&digest=%@&auth_token=%@", API_URL, model.orderNumber, model.digest, _authToken] parameters:nil progress:nil success:^(NSURLSessionTask *task, id responseObject) {
        if ([responseObject isKindOfClass:[NSDictionary class]]) {
            successCallback([ShowOrderResponse fromJson:(NSDictionary *) responseObject]);
        } else{
            failureCallback([PgwApi unableToParseResponse]);
        }
    } failure:^(NSURLSessionTask *operation, NSError *error) {
        failureCallback([PgwApi getErrorFromFailure:error]);
    }];
}

-(void) checkOrder:(ShowOrderModel *)model success:(void (^)(TransactionSuccess *))successCallback failure:(void (^)(ApiError *))failureCallback{
    AFHTTPSessionManager *manager = [AFHTTPSessionManager manager];
    [manager GET:[NSString stringWithFormat:@"%@/api/check-order?order_number=%@&digest=%@&auth_token=%@", API_URL, model.orderNumber, model.digest, _authToken] parameters:nil progress:nil success:^(NSURLSessionTask *task, id responseObject) {
        if ([responseObject isKindOfClass:[NSDictionary class]]) {
            successCallback([TransactionSuccess fromJson:(NSDictionary *) responseObject]);
        } else{
            failureCallback([PgwApi unableToParseResponse]);
        }
    } failure:^(NSURLSessionTask *operation, NSError *error) {
        failureCallback([PgwApi getErrorFromFailure:error]);
    }];
}

-(void) checkTransaction:(int)transactionId success:(void (^)(Transaction *))successCallback failure:(void (^)(ApiError *))failureCallback{
    AFHTTPSessionManager *manager = [AFHTTPSessionManager manager];
    [manager GET:[NSString stringWithFormat:@"%@/api/check-transaction-id/%d?auth_token=%@", API_URL, transactionId, _authToken] parameters:nil progress:nil success:^(NSURLSessionTask *task, id responseObject) {
        if ([responseObject isKindOfClass:[NSDictionary class]]) {
            successCallback([Transaction fromJson:(NSDictionary *) responseObject]);
        } else{
            failureCallback([PgwApi unableToParseResponse]);
        }
    } failure:^(NSURLSessionTask *operation, NSError *error) {
        failureCallback([PgwApi getErrorFromFailure:error]);
    }];
}

+(ApiError *) unableToParseResponse{
    return [ApiErrors libraryParsingError:@"Unable to parse response"];
}

+(ApiError *) unableToParseErrorResponse{
    return [ApiErrors libraryParsingError:@"Unknown error"];
}

+(ApiError *) getErrorFromFailure: (NSError *) error{
    NSString* errorResponse = [[NSString alloc] initWithData:(NSData *)error.userInfo[AFNetworkingOperationFailingURLResponseDataErrorKey] encoding:NSUTF8StringEncoding];
    if (errorResponse) {
        NSData *webData = [errorResponse dataUsingEncoding:NSUTF8StringEncoding];
        if (webData) {
            NSError *error;
            NSDictionary *data = [NSJSONSerialization JSONObjectWithData:webData options:0 error:&error];
            BusinessError* businessError = [BusinessError fromJson:data];

            if (businessError) {
                return businessError;
            }

            ApiError *apiError = [ApiError fromJson:data];
            if (apiError) {
                return apiError;
            }
        }
    }

    return [PgwApi unableToParseErrorResponse];
}

-(void) getError:(int)errorId success:(void (^)(TransactionErrorDetailed *))successCallback failure:(void (^)(ApiError *))failureCallback{
    AFHTTPSessionManager *manager = [AFHTTPSessionManager manager];
    [manager GET:[NSString stringWithFormat:@"%@/api/get-error/%d?auth_token=%@", API_URL, errorId, _authToken] parameters:nil progress:nil success:^(NSURLSessionTask *task, id responseObject) {
        if ([responseObject isKindOfClass:[NSDictionary class]]) {
            successCallback([TransactionErrorDetailed fromJson:(NSDictionary *) responseObject]);
        } else{
            failureCallback([PgwApi unableToParseResponse]);
        }
    } failure:^(NSURLSessionTask *operation, NSError *error) {
        failureCallback([PgwApi getErrorFromFailure:error]);
    }];
}

-(void)paymentMethods:(NSString *)userId success:(void (^)(PaymentMethodResponse *))successCallback failure:(void (^)(ApiError *))failureCallback {
    AFHTTPSessionManager *manager = [AFHTTPSessionManager manager];
    [manager GET:[NSString stringWithFormat:@"%@/api/payment-methods/list?id=%@&auth_token=%@", API_URL, userId, _authToken] parameters:nil progress:nil success:^(NSURLSessionTask *task, id responseObject) {
        if ([responseObject isKindOfClass:[NSDictionary class]]) {
            successCallback([PaymentMethodResponse fromJson:(NSDictionary *) responseObject]);
        } else{
            failureCallback([PgwApi unableToParseResponse]);
        }
    } failure:^(NSURLSessionTask *operation, NSError *error) {
        failureCallback([PgwApi getErrorFromFailure:error]);
    }];
}

- (void)deletePaymentMethod:(NSString *)paymentMethodId reason:(NSString *)reason success:(void (^)(DeletePaymentMethodResponse *))successCallback {
    AFHTTPSessionManager *manager = [AFHTTPSessionManager manager];
    NSDictionary *parameters = @{@"delete_reason": reason};
    [manager POST:[NSString stringWithFormat:@"%@/api/payment-method/delete?id=%@&auth_token=%@", API_URL, paymentMethodId, _authToken] parameters:parameters progress:nil success:^(NSURLSessionTask *task, id responseObject) {
        if ([responseObject isKindOfClass:[NSDictionary class]]) {

            successCallback([[DeletePaymentMethodResponse alloc] init:[DeletePaymentMethodSuccess fromJson:(NSDictionary *)responseObject] error:nil]);
        } else {
            successCallback([[DeletePaymentMethodResponse alloc] init:nil error:[[DeletePaymentMethodError alloc]init:@"not_deleted" message:@"Delete failed"]]);
        }
    } failure:^(NSURLSessionTask *operation, NSError *error) {
        successCallback([[DeletePaymentMethodResponse alloc] init:nil error:[[DeletePaymentMethodError alloc]init:@"not_deleted" message:@"Delete failed"]]);
    }];
}

- (void)updatePaymentMethod:(NSString *)paymentMethodId expiryDate:(NSString *)expiryDate success:(void (^)(UpdatePaymentMethodResponse *))successCallback {
    AFHTTPSessionManager *manager = [AFHTTPSessionManager manager];
    NSDictionary *parameters = @{@"expiry_date": expiryDate};
    [manager POST:[NSString stringWithFormat:@"%@/api/payment-method/update?id=%@&auth_token=%@", API_URL, paymentMethodId, _authToken] parameters:parameters progress:nil success:^(NSURLSessionTask *task, id responseObject) {
        if ([responseObject isKindOfClass:[NSDictionary class]]) {

            successCallback([[UpdatePaymentMethodResponse alloc] init:[UpdatePaymentMethodSuccess fromJson:(NSDictionary *)responseObject] error:nil]);
        } else {
            successCallback([[UpdatePaymentMethodResponse alloc] init:nil error:[[UpdatePaymentMethodError alloc]init:@"not_updated" message:@"Update failed"]]);
        }
    } failure:^(NSURLSessionTask *operation, NSError *error) {
        successCallback([[UpdatePaymentMethodResponse alloc] init:nil error:[[UpdatePaymentMethodError alloc]init:@"not_updated" message:@"Update failed"]]);
    }];
}

+(NSURLRequest *) payCvv:(NSString *)input{
    return [NSURLRequest requestWithURL:[NSURL URLWithString:[NSString stringWithFormat:@"%@/default/pay-cvv/%@", API_URL, input]]];
}

+(NSURLRequest *) addCard:(NSString *)input{
    return [NSURLRequest requestWithURL:[NSURL URLWithString:[NSString stringWithFormat:@"%@/default/add-card/%@", API_URL, input]]];
}

+(NSURLRequest *) addPaymentMethod:(NSString *)input{
    return [NSURLRequest requestWithURL:[NSURL URLWithString:[NSString stringWithFormat:@"%@/default/add-payment-method/%@", API_URL, input]]];
}

@end
