//
//  SuccessTransactionResolverRunnable.m
//  WebtehPaymentLib
//
//  Created by Jasmin Suljic on 04/10/2018.
//

#import <Foundation/Foundation.h>
#import "SuccessTransactionResolverRunnable.h"
#import "RedirectResolver.h"

@interface SuccessTransactionResolverRunnable ()
@property(nonatomic, weak) id<TransactionLifecycleAdapterDelegate> delegate;
@end

@implementation SuccessTransactionResolverRunnable

- (id)init:(id<TransactionLifecycleAdapterDelegate>)delegate {
    _delegate = delegate;
    return [self init];
}

- (void)run:(int)resolvedCode request:(NSString *)request redirectResolver:(RedirectResolver *)redirectResolver inputParam:(NSString *)inputParam {
    // If transaction is not in progress there is no need for transaction success handling
    // This is guard from multiple delegate calls when timeout is invoked and webview is not stopped.
    // After a while transaction success or transaction failure occurs.
    if (!redirectResolver.transactionInProgress) {
        [redirectResolver logTransactionInfo:[NSString stringWithFormat:@"Transaction success invoked, but transactionInProgress = false. Ignoring."]];
        return;
    }
    
    redirectResolver.transactionInProgress = false;
    [redirectResolver stopTimeoutTracking];

    [_delegate hideProgress];

    TransactionSuccess *success = [TransactionSuccess fromString:inputParam];
    [redirectResolver logTransactionInfo:[NSString stringWithFormat:@"Transaction success %@", success]];
    [_delegate transactionSuccess:success];
}

@end
