//
//  ThreeDSResolverRunnable.m
//  WebtehPaymentLib
//
//  Created by Jasmin Suljic on 04/10/2018.
//

#import <Foundation/Foundation.h>
#import "ThreeDSResolverRunnable.h"
#import "RedirectResolver.h"

@interface ThreeDSResolverRunnable ()
@property(nonatomic, weak) id<SharedLifecycleDelegate> delegate;
@end

@implementation ThreeDSResolverRunnable

- (id)init:(id<SharedLifecycleDelegate>)delegate {
    _delegate = delegate;
    return [self init];
}

- (void)run:(int)resolvedCode request:(NSString *)request redirectResolver:(RedirectResolver *)redirectResolver inputParam:(NSString *)inputParam {
    // If transaction is not in progress there is no need for transaction success handling
    // This is guard from multiple delegate calls when timeout is invoked and webview is not stopped.
    // After a while transaction success or transaction failure occurs.
    if(!redirectResolver.transactionInProgress){
        [redirectResolver logTransactionInfo:[NSString stringWithFormat:@"Transaction 3ds invoked, but transactionInProgress = false. Ignoring."]];
        return;
    }
    redirectResolver.transactionInProgress = false;
    [redirectResolver stopTimeoutTracking];
    [_delegate hideProgress];
    
    if ([_delegate respondsToSelector:@selector(threeDsLoaded)]) {
        [redirectResolver logTransactionInfo:[NSString stringWithFormat:@"_transactionLifecycleAdapter.delegate#threeDsLoaded"]];
        [_delegate threeDsLoaded];
    }
}

@end
