//
//  TimeoutResolverRunnable.m
//  WebtehPaymentLib
//
//  Created by Jasmin Suljic on 04/10/2018.
//

#import <Foundation/Foundation.h>
#import "TimeoutResolverRunnable.h"
#import "RedirectResolver.h"

@interface TimeoutResolverRunnable ()
@property(nonatomic, weak) id<SharedLifecycleDelegate> delegate;
@property (nonatomic, copy) void (^onTimeout)(void);
@end

@implementation TimeoutResolverRunnable

- (id)init:(id<SharedLifecycleDelegate>)delegate onTimeout:(void (^)(void))onTimeout {
    _delegate = delegate;
    _onTimeout = onTimeout;
    return [self init];
}

- (void)run:(int)resolvedCode request:(NSString *)request redirectResolver:(RedirectResolver *)redirectResolver inputParam:(NSString *)inputParam {
    // If transaction is not in progress there is no need for transaction success handling
    // This is guard from multiple delegate calls when timeout is invoked and webview is not stopped.
    // After a while transaction success or transaction failure occurs.
    if(!redirectResolver.transactionInProgress){
        [redirectResolver logTransactionInfo:[NSString stringWithFormat:@"Transaction timeout invoked, but transactionInProgress = false. Ignoring."]];
        return;
    }
    
    redirectResolver.transactionInProgress = false;
    [redirectResolver stopTimeoutTracking];
    
    _onTimeout();
}

@end
